unit PM_CommFunction;

interface

Uses Windows, Registry, SysUtils, Variants, Classes, PM_CommData;

  function HexToStr(hex: string):string;
  function RegSessionToStr(tx: string):string;
  function BArrayToStr(b: TArrayOfBool):string;
  function StrToBArray(s: string):TArrayOfBool;
  procedure GetSessionPUTTY;
  procedure GetSessionRDP;
  function CreateID(val:integer):string;
  function SessionExist(rs, pr: string):boolean;
  function FindCommandSchema(p:string): TExecCommand;
  function ChangeFields(s: string; h: THostRecord): string;
  function CreateCommand(p: string; h: THostRecord): TExecCommand;
  function ReplaceString(S, S1, S2: string): string;
  function CutOptionalChain(s: string): string;
  function LastPos(sub,s: string): integer;
  function FindIconIndex(i: integer):integer;
  function IPToSort(ip: string):string;
  function DomainToSort(ip: string):string;


implementation


function HexToStr(hex:string):string;
begin
  result:=Chr(StrToInt('$'+UpperCase(hex)));
end;

function RegSessionToStr(tx:string):string;
var
  i: integer;
  S: string;
begin
  for i:=length(tx) downto 1 do if tx[i]='%' then
  begin
    S:=HexToStr(copy(tx,i+1,2));
    delete(tx,i,3); insert(S,tx,i);
  end;
  result:=tx;
end;

function BArrayToStr(b: TArrayOfBool):string;
var
  i: integer;
begin
  result:='';
  for i:=0 to High(b) do
    if b[i] then result:=result+'1' else result:=result+'0';
end;

function StrToBArray(s: string):TArrayOfBool;
var
  i: integer;
begin
  for i:=0 to High(result) do
    if copy(s,i+1,1)='1' then result[i]:=True else result[i]:=False;
end;

procedure GetSessionPUTTY;
var
  Reg: TRegistry;
  i: integer;
  Session: TStringList;
begin
  Session:=TStringList.Create;
  Session.Clear;
  Reg:= TRegistry.Create;
  try
    Reg.RootKey:= HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(PuttyPath) then
      if Reg.HasSubKeys then Reg.GetKeyNames(Session);
  finally
    Reg.Free;
  end;
  SetLength(SessionPUTTY,Session.Count);
  for i:=0 to Session.Count-1 do SessionPUTTY[i]:=Session[i];
  Session.Free;
end;

procedure GetSessionRDP;
var
  i: integer;
  SR: TSearchRec;
  Session: TStringList;
begin
  Session:=TStringList.Create;
  Session.Clear;
  i:=FindFirst('*.rdp', faAnyFile, SR);
  while i=0 do
  begin
    Session.Append(SR.Name);
    i:=FindNext(SR);
  end;
  FindClose(SR);
  SetLength(SessionRDP,Session.Count);
  for i:=0 to Session.Count-1 do SessionRDP[i]:=Session[i];
  Session.Free;
end;

function CreateID(val: integer): string;
begin
  result:='PM'+Format('%.6d',[val]);
end;

function FindCommandSchema(p:string): TExecCommand;
var
  i: integer;
begin
  result.Exec:=''; result.Option:='';
  for i:=0 to High(ExecCommands) do with ExecCommands[i] do
    if UpperCase(p)=UpperCase(Protocol) then
    begin
      result.Exec:=Exec;
      result.Option:=Option;
    end;
end;

function ChangeFields(s: string; h: THostRecord): string;
begin
  result:=ReplaceString(s,'%ip%', h.IP);
  result:=ReplaceString(result,'%protocol%', LowerCase(h.Protocol));
  result:=ReplaceString(result,'%user%', h.User);
  result:=ReplaceString(result,'%session%', h.Session);
  if h.Port>0 then
    result:=ReplaceString(result,'%port%', IntToStr(h.Port)) else result:=ReplaceString(result,'%port%', '');
  result:=ReplaceString(result,'%path%', h.Path);
  if h.Verbosity then
    result:=ReplaceString(result,'%verbosity%', ' ') else result:=ReplaceString(result,'%verbosity%', '');
end;

function LastPos(sub,s: string): integer;
begin
  result:=-1;
  while pos(sub,s)>0 do
  begin
    result:=pos(sub,s);
    delete(s,result,1);
    insert(' ',s,result);
  end;
end;

function CutOptionalChain(s: string): string;
var
  l, e: integer;
begin
  while pos(']',s)>0 do
  begin
    e:=pos(']',s);
    l:=LastPos('[',copy(s,1,e-1));
    if pos(CommandErr,copy(s,l,(e-l)))>0 then
      delete(s,l,(e-l)+1) else
      begin
        delete(s,e,1);
        delete(s,l,1);
      end;
  end;
  result:=s;
end;

function CreateCommand(p: string; h: THostRecord): TExecCommand;
var
  c: TExecCommand;
begin
  c:=FindCommandSchema(p);
  c.Exec:=ChangeFields(c.Exec, h);
  c.Option:=ChangeFields(c.Option, h);
  c.Exec:=CutOptionalChain(c.Exec);
  c.Option:=CutOptionalChain(c.Option);
  result:=c;
end;

function ReplaceString(S, S1, S2: string): string;
// zamienia wystpienie S1 na S2
var
  I: integer;
begin
  if s2='' then s2:=CommandErr;
  I:=pos(S1, S);
  if I>0 then
  begin
    delete(S,I,length(S1));
    insert(S2,S,I);
  end;
  result:=S;
end;

function SessionExist(rs, pr: string):boolean;
// prawda jeli zapamietana sesja ma plik konfiguracyjny (rdp) lub zapis w regu
// jesli sesja nie zostala lub nie powinna byc zdefinowana -> prawda
// rs - sesja
// pr - protok: rdp, itd.
var
  i, j:integer;
begin
  result:=false;
  rs:=LowerCase(Trim(rs)); pr:=LowerCase(Trim(pr));
  if rs='' then result:=true;
  for i:=0 to High(Protocols) do
    if (LowerCase(Protocols[i].Name)=pr) and (Protocols[i].Session=0) then result:=true;
  if (rs<>'') then
  begin
    for j:=0 to High(Protocols) do
    begin
      if (LowerCase(Protocols[j].Name)=pr) and (LowerCase(Protocols[j].SavedSession)='putty') then
        for i:=0 to High(SessionPUTTY) do
          if rs=LowerCase(RegSessionToStr(SessionPUTTY[i])) then result:=true;
      if (LowerCase(Protocols[j].Name)=pr) and (LowerCase(Protocols[j].SavedSession)='rdp') then
        for i:=0 to High(SessionRDP) do
          if rs=LowerCase(SessionRDP[i]) then result:=true;
    end;
  end;
end;

function FindIconIndex(i: integer):integer;
var
  j: integer;
begin
  result:=0;
  for j:=0 to high(DevIcons) do
    if DevIcons[j].id=i then result:=j+1;
end;

function IPToSort(ip: string):string;
var
  isIP: bool;
  idx: integer;
  i: integer;
  iporg: string;
begin
  iporg:=ip;
  isIP:=true;
  result:='';
  for i:=1 to length(ip) do
    if not(ip[i] in ['.','0','1','2','3','4','5','6','7','8','9']) then isIP:=false;
  while(isIP and (pos('.',ip)>1)) do
  begin
    idx:=(pos('.',ip));
    if idx>4 then isIP:=false;
    if idx=2 then ip:='00'+ip;
    if idx=3 then ip:='0'+ip;
    if idx<=4 then
    begin
      result:=result+copy(ip,1,3);
      ip:=copy(ip,5,1024);
    end;
  end;
  if length(ip)>3 then isIP:=false;
  if isIP then
  begin
    if length(ip)=1 then ip:='0'+ip;
    if length(ip)=2 then ip:='0'+ip;
    result:=result+ip;
  end else result:=DomainToSort(iporg);
end;

function DomainToSort(ip: string):string;
begin
  result:='';
  while(pos('.',ip)>1) do
  begin
    result:=copy(ip,1,pos('.',ip))+result;
    ip:=copy(ip,pos('.',ip)+1,1024);
  end;
  result:=ip+'.'+result;
end;

end.
